<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use Illuminate\Support\Facades\Session;
use App\Managers\LiveProposal\LiveProposalManager;
use App\Managers\LiveProposalLikeCount\LiveProposalLikeCountManager;
use App\Validators\LiveProposalValidator;
use App\Managers\Profile\ProfileManager;
use App\Managers\Paypal\PaypalManager;
use App\Jobs\LiveProposalEmail;
use App\Managers\User\UserManager;
use Illuminate\Support\Facades\Mail;

class LiveProposalApiController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->liveProposalManager = new LiveProposalManager();
        $this->liveProposalLikeCountManager = new LiveProposalLikeCountManager();
        $this->liveProposalValidator = new LiveProposalValidator();
        $this->profileManager = new ProfileManager();
        $this->paypalManager = new PaypalManager();
        $this->profileInfo = $this->profileManager->first();
        $this->userManager = new UserManager();
    }
    
    public function addLiveProposal(Request $request)
    {
        if(!$request->get('data'))
        {
            return response()->json(['No input found'], 400);
        }
        
        $data = $request->get('data');
        
        $validation = $this->liveProposalValidator->add($data, $this->profileInfo->live_proposal_limit);
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        $data['user_id'] = Session::get('general');
        $data['UUID'] = $this->helper->addUuid();
        $data['expiry_date'] = date("Y-m-d H:i:s",(strtotime('+' . $this->profileInfo->live_proposal_limit . ' hour' ,strtotime(date("Y-m-d H:i:s")))));
        $proposal = $this->liveProposalManager->add($data);
        if(!$proposal)
        {
            return response()->json(['Something went wrong'], 400);
        }
        
        //$users = $this->userManager->where('type','general')->where('active', 1)->get();
        //dispatch(new LiveProposalEmail($users));


        $emailArray = array();
        $users = $this->userManager->where('users.type','general')->join('paypal', 'users.UUID', '=', 'paypal.user_id')->where('users.active', 1)->where('paypal.expiry_date', '>=', date("Y-m-d"))->get();

        foreach($users AS $user)
        {
            $emailArray[] = trim($user->email);
        }

        Mail::send('emails.live-proposal-new', ['text' => 'view', 'url' => url('/')], function($message) use ($emailArray)
        {  
            $message->from('info@realclubmanager.com', 'SUPPORTER 2.0');
            $message->to('contact@realclubmanager.com');
            
            $message->bcc($emailArray);
            $message->subject('Propunere LIVE noua');    
        });
        return response()->json(['Live Proposal Added'], 200);
    }
    
    public function activeLiveProposals(Request $request)
    {
        $res['votes'] = $this->paypalManager->getCurrentUserTotalVotesCount($request->session()->get('general'))['total'];
        $totalVotesOfAllUsers = $request->session()->get('total_votes');
        if(!is_numeric($totalVotesOfAllUsers))
        {
            $totalVotesOfAllUsers = 0;
        }

        $res['totalMaxAllowedVotes'] = ceil($totalVotesOfAllUsers  * 30 / 100);
            
        $liveProposals = $this->liveProposalManager->with('likes', 'user')->where('reject', 0)->where('expiry_date', '>=', date('Y-m-d'))->orderBy('created_at', 'DESC')->get();
        $count = 0;
        foreach($liveProposals AS $liveProposal)
        {
            $likeCounts = $this->liveProposalLikeCountManager->getTotalLikesByUser($liveProposal['UUID'], $request->session()->get('general'));
            $res['tactics'][$count] = $liveProposal;
            $res['tactics'][$count]['max_likes'] = $likeCounts->total_likes;
            $count++;
        }
        
        return response()->json($res,200);
    }
    
     public function likeLiveProposal(Request $request)
    {
        if(!$request->get('noOfLikes') || !$request->get('id'))
        {
            return response()->json(['Something went wrong'], 400);
        }
        $id = $request->get('id');
        
        $noOfLikes = $request->get('noOfLikes');
        $likeCounts = $this->liveProposalLikeCountManager->getTotalLikesByUser($id, $request->session()->get('general'));
        $totalUserVotes = $this->paypalManager->getCurrentUserTotalVotesCount($request->session()->get('general'));
        
        
        
        $totalVotesOfAllUsers = $request->session()->get('total_votes');
        if(!is_numeric($totalVotesOfAllUsers))
        {
            $totalVotesOfAllUsers = 0;
        }
        
        $totalMaxAllowedVotes = ceil($totalVotesOfAllUsers  * 30 / 100);
        if($noOfLikes > $totalMaxAllowedVotes)
        {
            return response()->json(['message' => 'You donot have enough votes to vote'], 403);
            exit;
        }
        
        
        
        if($totalVotesOfAllUsers < $noOfLikes)
        {
            return response()->json(['You donot have enough votes to vote'], 403);
            exit;
        }
        
        if(($totalMaxAllowedVotes - $likeCounts['total_likes']) < $noOfLikes)
        {
            return response()->json(['You donot have enough votes to vote'], 403);
            exit;
        }
        
        $validation = $this->liveProposalValidator->like($noOfLikes, $id);
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $add = $this->liveProposalLikeCountManager->addLike($id, $noOfLikes);
        if(!$add)
        {
            return response()->json(['Something went wrong'], 400);
            exit;
        }
        return response()->json(['Succesfully supported'], 200);
    }
}